package com.pwt.web.service;

import java.util.List;

import com.pwt.web.domain.ChangePowerModel;
import com.pwt.web.domain.ExchangeStatistics;
import com.pwt.web.domain.PwtChangeBatteryRecord;

/**
 * 换电记录Service接口
 * 
 * @author linyun
 * @date 2022-11-17
 */
public interface IPwtChangeBatteryRecordService 
{
    /**
     * 查询换电记录
     * 
     * @param recordId 换电记录ID
     * @return 换电记录
     */
    public PwtChangeBatteryRecord selectPwtChangeBatteryRecordById(Long recordId);

    /**
     * 查询换电记录列表
     * 
     * @param pwtChangeBatteryRecord 换电记录
     * @return 换电记录集合
     */
    public List<PwtChangeBatteryRecord> selectPwtChangeBatteryRecordList(PwtChangeBatteryRecord pwtChangeBatteryRecord);

    /**
     * 新增换电记录
     * 
     * @param pwtChangeBatteryRecord 换电记录
     * @return 结果
     */
    public int insertPwtChangeBatteryRecord(PwtChangeBatteryRecord pwtChangeBatteryRecord);

    /**
     * 修改换电记录
     * 
     * @param pwtChangeBatteryRecord 换电记录
     * @return 结果
     */
    public int updatePwtChangeBatteryRecord(PwtChangeBatteryRecord pwtChangeBatteryRecord);

    /**
     * 批量删除换电记录
     * 
     * @param recordIds 需要删除的换电记录ID
     * @return 结果
     */
    public int deletePwtChangeBatteryRecordByIds(Long[] recordIds);

    /**
     * 删除换电记录信息
     * 
     * @param recordId 换电记录ID
     * @return 结果
     */
    public int deletePwtChangeBatteryRecordById(Long recordId);

    /**
     * @Description: 查询日换电量/换电次数
     * @Author: linyun
     * @Date: 2022/11/23 15:31
     * @Return:java.util.List<com.pwt.web.domain.ChangePowerModel>
     **/
    public List<ChangePowerModel> getDayDataInfo();

    /**
     * @Description: 查询月换电量/换电次数
     * @Author: linyun
     * @Date: 2022/11/23 15:31
     * @Return:java.util.List<com.pwt.web.domain.ChangePowerModel>
     **/
    public List<ChangePowerModel> getMonthDataInfo();

    /**
     * @Description: 查询年换电量/换电次数
     * @Author: linyun
     * @Date: 2022/11/23 15:31
     * @Return:java.util.List<com.pwt.web.domain.ChangePowerModel>
     **/
    public List<ChangePowerModel> getYearDataInfo();

    public ExchangeStatistics getExchangeStatistics();

    int getChangeCount(String batterySn);

    PwtChangeBatteryRecord getLastestRecord();
}
